const jails = {
	"south" : {
		jail: "CastleCrypt",
		outroom: "Thrones",
		ranks: ["Imperial Knight", "Knight Captain", "Emperor", "Empress", "Prince", "Princess", "Emperor Helper", "Emperor Regent", "Empress Regent", "Administrator", "Moderator"],
		condition: (client) => {
			return client.data.rank >= 3
		}
	},
	"dorval" : {
		jail: "SunderJail",
		outroom : "Sunderhouse",
		ranks: ["Dorval Knight", "King of Dorval", "Queen of Dorval", "Administrator"],
		condition: (client) => {
			return client.data.rank >= 4
		}
	},
	"providence" : {
		jail: "DeadMansDungeons",
		outroom: "DeadMansGuts",
		ranks: ["Cavalier", "Top Dogg", "Administrator"],
		condition: (client) => {
			return client.data.rank >= 4
		}
	},
	"axel" : {
		jail: "AxelDungeon",
		outroom: "EarthHall",
		ranks: ["Axel Knight", "King of Axel", "Queen of Axel", "Administrator"],
		condition: (client) => {
			return client.data.rank >= 4
		}
	},
	"baraddur" : {
		jail: "BaraddurDungeons",
		outroom: "BaraddurCouncilChamber",
		ranks: ["Black Captain"],
		condition: (client) => {
			return client.getItemCount("BlackKey") > 0;
		}
	},
	"cobia" : {
		jail: "CoboDungeon",
		outroom: "Cobofort",
		ranks: ["Cobo Knight", "King of Cobia", "Queen of Cobia", "Administrator"],
		condition: (client) => {
			return client.data.rank >= 4
		}
	},
	"gondolin" : {
		jail: "GondolinJail",
		outroom: "Gondolin",
		ranks: ["Gondolin Knight", "King of Gondolin", "Queen of Gondolin", "Administrator"],
		condition: (client) => {
			return client.data.rank >= 4
		}
	},
	"menegroth": {
		jail: "Menegroth",
		outroom: "Menegroth",
		condition: (client) => {
			return client.data.character == "Spider"
		}
	},
	"goblintown": {
		jail: "GoblinTown",
		outroom: "GoblinTown",
		condition: (client) => {
			return client.prop("goblinking") || client.prop("goblin");
		}
	},
	"vinyamar": {
		jail: "VinyamarJail",
		outroom: "VinyamarCastle",
		ranks: ["King of Vinyamar", "Queen of Vinyamar", "Vinyamar", "Emperor"],
	}
}
const canUseJail = (client, jail) => {
	// If ranks are defined, loop through each rank
	// if the user has that rank in their title, return true
	// If not, check the condition function if their client matches 

	 if(jail.ranks) {
		for(let rank of jail.ranks) {
			if(client.data.title.toLowerCase().contains(rank.toLowerCase())) {
				return true;
			}
		}
	}
	if(jail.condition) {
		return jail.condition(client);
	}
}
const getIslandJail = (jailName) => {
	for(let island in jails) {
		if(jails[island].jail == jailName)
			return {island: island, jail: jails[island]};
	}
}

P3D.on("/arrest", (client, args) => {
	// When someone use the command e.g. /arrest damen south
	// args will contain: ['damen', 'south']
	// so i want to extract the right value into a var
	// e.g. let username = args.shift() <-- this will be 'damen'

	const username = args.shift()
	let island = args.shift()
	let hours = Number(args.shift()) || 1;
	let reason = args.join(" ") || "No reason given";

	
	if(hours < 0.5)
		hours = 0.5;

	if(island != undefined)
		island = island.toLowerCase()
	else
		return client.chat("Please specify a jail e.g. /arrest username south");

	const jail = jails[island];

	if(!jail)
		return client.chat(`${island} does not have a jail.`);

	if(!canUseJail(client, jail))
		return client.chat("You can't use this jail.");

	const sclient = getClientByName(username);
	
	if(isSecret(sclient) || !valid(sclient))
		return client.chat(`${username} is offline`);

	if(sclient.prop("unjailable"))
		return; // no feedback
	
		// about here, check if theyre already arrested
	if(sclient.data.jail)
		return client.chat("You can't arrest an user who is already in jail")

	if(sclient.roomId != client.roomId || DistanceVector(sclient.position, client.position) > 20)
		return client.chat("You must be very close to them to arrest them");
	
	if(client == sclient)
		return client.chat("You can't arrest yourself");
	
	// ARREST THE SCLIENT
	const robberShirt = "RobberShirt";

	sclient.data.jail = jail.jail;
	sclient.data.autorelease = time() + (hours * 3600);
	
	sclient.data.head = "";
	sclient.data.face = "";
	sclient.data.neck = "";
	sclient.data.body = robberShirt;
	sclient.data.feet = "";
	sclient.data.hands = "";
	
	// If they dont have the robber shirt, add it to their acc
	if(!sclient.getItemCount(robberShirt))
		sclient.addItem(robberShirt);    

	mysql.query("UPDATE users SET jail = ?, head = '', face = '', neck = '', body = ?, feet = '', hands = '', autorelease = ? WHERE id = ?", [sclient.data.jail, robberShirt, sclient.data.autorelease, sclient.data.id]);
	
	ModLog(Sprint("?:? arrested ?:? to ? for ? hours", [client.data.username, client.ip(), sclient.data.username, sclient.ip(), sclient.data.jail, hours]));
	
	mysql.query("INSERT INTO arrestlog (`arrester`, `arrested`, `jail`, `time`, `release`, `reason`) VALUES (?, ?, ?, ?, ?, ?)", [client.data.username, sclient.data.username, sclient.data.jail, time(), sclient.data.autorelease, reason]);
	
	sclient.joinRoom(sclient.data.jail);
	
	sclient.chat(`You have been thrown in jail for ${hours} hour(s). Rot in cell scum!`);
	client.room.chat(`${sclient.data.username} was thrown in jail for ${hours} hour(s).`);
});

P3D.on("/release", (client, args) => {
	const username = args.shift()
	const sclient = getClientByName(username)

	if(!valid(sclient) || isSecret(sclient))
		return client.chat("That user is not online")

	if(!sclient.data.jail)
		return client.chat("You can't release an user who isn't arrested")
	
	if(client == sclient)
		return client.chat("You can't release yourself")
	
	let jailRoom = sclient.data.jail;
	let islandJail = getIslandJail(jailRoom);
	
	if(!islandJail) 
		islandJail = {island: "south", jail: jails["south"]};
	
	let island = islandJail.island;
	let jail = islandJail.jail;

	if(!canUseJail(client, jail))
		return client.chat("You can't release from this jail");
	
	sclient.data.jail = "";
	sclient.data.autorelease = 0;

	sclient.joinRoom(jail.outroom);

	client.chat(`Released ${sclient.data.username} from ${island} jail`);
	sclient.chat("You have been released from jail. Welcome back to civilization.");
	
	mysql.query("UPDATE users SET jail = '', autorelease = 0 WHERE id = ?", [sclient.data.id]);
	mysql.query("UPDATE arrestlog SET `releasedBy` = ? WHERE `arrested` = ? AND `releasedBy` = ''", [client.data.username, sclient.data.username]);
})
