P3D.Fish = {
	"SeaBass": {
		fullName: "Sea Bass",
		scale: Vector3(1.2, 1.2, 1.2),
		speed: 30
	},
	"ParadiseFish": {
		fullName: "Paradise Fish",
		scale: Vector3(1, 1, 1),
		speed: 40
	},
	"BluefinTuna": {
		fullName: "Bluefin Tuna",
		scale: Vector3(1.1, 1.1, 1.1),
		speed: 30
	},
	"RadiumFish": {
		fullName: "Radium Fish",
		scale: Vector3(1.1, 1.1, 1.1),
		speed: 30
	}
};

P3D.FishInterval = function() {
	for(let i in rooms) {
		let room = rooms[i];
		
		if(!room) continue;
		if(!room.fishing) continue;
		
		// Loop through fishing points in room
		for(let e in room.fishing) {
			let fpoint = room.fishing[e];
			
			if(!fpoint) continue;
			
			let fishes = room.getFish(fpoint.name);
			
			for(let o in fishes) {
				if(!fishes[o]) continue;
				ManageFish(fishes[o], room);
			}
			
			if(fishes.length >= fpoint.maxFish) continue;
			if(rand(0, 1000) < 800) continue;
			
			if(!fpoint.position.vadd) {
				fpoint.position = Vector3(fpoint.position.x, fpoint.position.y, fpoint.position.z);
			}
			
			let fishPick = fpoint.fishes[rand(0, fpoint.fishes.length-1)];
			let fishPrefab = P3D.Fish[fishPick];
			
			if(!fishPrefab) continue;
			
			fishPrefab.name = fishPick;
			
			let fishId = "Fish_" + fishPrefab.name;
			let fishName = "Fish_" + rand(0, 10000);
			let positionSpawn = fpoint.position.vadd(Vector3(
				rand(-fpoint.radius, fpoint.radius),
				rand(-0.4, 0.4),
				rand(-fpoint.radius, fpoint.radius)
			));
			let rotationSpawn = Vector3(0, rand(0, 360), 0);
			
			let fish = Instantiate(fishId, fishName, room, positionSpawn, rotationSpawn, fishPrefab.scale, true);
			
			fish.isFish = true;
			fish.fpoint = fpoint;
			fish.prefab = fishPrefab;
		}
	}
}
ManageFish = function(fish, room) {
	let fpoint = fish.fpoint;
	
	if((time() - fish.lastTurn > fish.journeyLength || !fish.lastTurn) && !fish.attracted) {
		let facing = fpoint.position.vadd(Vector3(
			rand(-fpoint.radius, fpoint.radius),
			rand(-0.4, 0.4),
			rand(-fpoint.radius, fpoint.radius)
		));
		
		let chanceAttract = rand(0, 1000) > 800;
		
		if(chanceAttract) {
			let choices = [];
			for(let i in room.clients) {
				let sclient = room.clients[i];
				if(!valid(sclient)) continue;
				if(!sclient.fishing) continue;
				if(sclient.fishing.point.name != fpoint.name) continue;
				if(sclient.fishing.fishOnHook) continue;
				if(sclient.fishing.inboundFish) continue;
				choices.push(sclient);
			}
			if(choices.length > 0) {
				let choice = choices[rand(0, choices.length-1)];
				facing = choice.fishing.hook;
				//Instantiate("Ins_Spawn", "spawntest", room, facing, Vector3(0,0,0), Vector3(1,1,1), false);
				fish.attracted = choice;
				fish.attractedTime = time();
				fish.arrivedTime = 0;
				choice.fishing.inboundFish = fish;
			}
		}
		
		room.send("fshf", [fish.name, fish.speed || fish.prefab.speed, facing.x, facing.y, facing.z]);
		
		fish.lastTurn = time();
		fish.facing = facing;
		fish.totalDistance = DistanceVector(fish.position, fish.facing);
		fish.journeyLength = fish.totalDistance * (1.6 * (30 / (fish.speed || fish.prefab.speed)));
		fish.speed = fish.prefab.speed;
		fish.startPos = fish.position;
	}
	if(fish.attracted) {
		if(fish.attracted.fishing) {
			let pos = Vector3(fish.position.x, fish.attracted.fishing.hook.y, fish.position.z);
			let dist = DistanceVector(fish.attracted.fishing.hook, pos);
			if(dist < 0.1 && !fish.arrivedTime) {
				fish.arrivedTime = time();
				fish.attracted.fishing.fishOnHook = fish;
				fish.attracted.alert("Fish on hook! Click Click Click!");
			}
			if((fish.attracted.fishing.pulls < 152 && fish.attracted.fishing.pulls > 50) && (time() - fish.arrivedTime > 5 && fish.arrivedTime) && (rand(0, 1000) < 900)) {
				room.send("unplace", [fish.name]);
				fish.attracted.alert(`<color=#FFD700>Catch! Its a ${fish.prefab.fullName}!</color>`);
				fish.attracted.addItem(fish.prefab.name);
				fish.attracted.fishing.pulls = 0;
				fish.attracted.fishing.clicks = 0;
				fish.attracted.fishing.totals = 0;
				fish.attracted.fishing.fishOnHook = null;
				fish.attracted.fishing.inboundFish = null;
				fish.attracted.fishing.lastPull = 0;
				
				delete room.insMap[fish.name];
				fish = null;
				return;
			}
		}
		if((rand(0, 1000) > 100 && (time() - fish.arrivedTime > 10 && fish.arrivedTime)) || time() - fish.attractedTime > 60 || !fish.attracted.fishing) {
			if(fish.attracted.fishing) {
				fish.attracted.fishing.lastPull = 0;
				fish.attracted.fishing.pulls = 0;
				fish.attracted.fishing.totals = 0;
				fish.attracted.fishing.clicks = 0;
				fish.attracted.fishing.fishOnHook = null;
				fish.attracted.fishing.inboundFish = null;
				fish.attracted.alert("Oops! It escaped!");
			}
			fish.attracted = null;
			fish.speed = fish.prefab.speed * 4;
			fish.lastTurn = 0;
			fish.arrivedTime = 0;
			fish.attractedTime = 0;
			return;
		}
	}
	
	let timeFinish = (fish.lastTurn + fish.journeyLength);
	
	let coefficient = reverseNumber((timeFinish - time()) / fish.journeyLength, 0, 1);
	
	if(coefficient > 1) coefficient = 1;
	if(coefficient < 0) coefficient = 0;
	
	fish.position = fish.startPos.lerp(fish.facing, coefficient);
	
	if(time() - fish.lastSync > 1 || !fish.lastSync) {
		room.send("fshs", [fish.name, fish.position.x.toFixed(3), fish.position.y.toFixed(3), fish.position.z.toFixed(3)]);
		fish.lastSync = time();
	}
	
	//Instantiate("Ins_Spawn", "fpostest", room, fish.position, Vector3(0,0,0), Vector3(0.5,0.5,0.5), false);
}

P3D.on("echo", function(client, echoId, params) {
	if(echoId != "GetFishTank") return;
	
	let fid = params.shift();
	
	if(!fid) return;
	
	let exists = false;
	
	for(let i in client.room.fmap) {
		if(!client.room.fmap[i]) continue;
		if(client.room.fmap[i].fid == fid) {
			exists = true;
			break;
		}
	}
	
	if(!exists) {
		//client.chat(fid + " fish tank doesn't exist.");
		return;
	}
	
	RegisterContainer(fid, client.room, function(container) {
		if(!container) {
			//client.chat(fid + " does not have a container.");
			return;
		}
		client.send("cont", [container.id, container.name, 2]);
	});
});

clearInterval(P3D.FishInterval);
P3D.FishInterval = setInterval(P3D.FishInterval, 500);