const bombPosition = Vector3(79, 10, 130);       // Position of bomb in world space (x,y,z)
const bombStart = 1665781200;            // Unix timestamp which says when the bomb will start ticking down.
const bombLifespan = 3600;                   // 1 hour in seconds
const bombRoom = rooms["Island"];            // the room where the bomb is in
const bombDeactivationCode = "sriram";       // De-activation code

/*
	This bombUpdater function will run every 10 seconds.
	It will say how long the bomb has until it blows up.
*/
const bombUpdater = () => {
	if(time() < bombStart || bombRoom.bombBlownUp || bombRoom.bombDeactivated) 
		return; // bomb is not active yet OR its already blown up OR bomb is deactivated!
	
	const bombBlowsUpAt = bombStart + bombLifespan;
	const secondsLeft = bombBlowsUpAt - time();
	
	if(secondsLeft < 0 && !bombRoom.bombBlownUp) {
		explosion();
		bombRoom.bombBlownUp = true;
	}
	
	bombRoom.chat(`${P3D.GetTimePlayed(secondsLeft)} left until bomb explodes!`);
}

/*
	Function that handles the bomb panel being activated in-game.
	
		1) It shows an input box
		2) If the code is right, it turns off the bomb
		3) If the code is wrong, it sends a warning message
*/
const activateBombPanel = (client) => {
	if(bombRoom.bombDeactivated)
		return client.notice(`Bomb is already de-activated by ${bombRoom.bombDeactivatedBy}`);
	
	client.input("ENTER DE-ACTIVATION CODE", (message, client) => {
		let correct = [bombDeactivationCode];
		
		let answer = String(message).toLowerCase().trim();
		
		for(let i in correct) {
			if(answer == correct[i]) {
				// CORRECT ANSWER!
				deactivateBomb(client);
				return;
			}
		}
		
		bombRoom.chat(`${client.data.username} has entered an incorrect de-activation code!`);
		return client.notice("INCORRECT");
	});
}

/*
	Explosion function which makes the explosion appear
*/
const explosion = () => {
	const scale = 4;
	
	Instantiate("ExplosionBomb", "bombexplosion", bombRoom, bombPosition, Vector3(0,0,0), Vector3(scale,scale,scale));
	
	ApplyRadiusDamage(
		bombPosition, 
		200, 
		3000,
		bombRoom
	);
}

/*
	Deactivate function to turn bomb off
	* Only called when the user gets the de-activation code correct *
*/
const deactivateBomb = (client) => {
	bombRoom.bombDeactivated = true;
	bombRoom.bombDeactivatedBy = client.data.username;
	
	client.room.chat(`<color#FFD700>${client.data.username} has de-activated the bomb!</color>`);
}

/*
	Setup server events
*/
P3D.on("tool", (client, tool, pos) => {
	if(tool != "bomb") return;
	
	activateBombPanel(client);
}, "bomb");

P3D.on("/bombpanel", (client, args) => {
	activateBombPanel(client);
});

// Alias for bomb updater so function can be updated without server restart
P3D.BombUpdate = bombUpdater;

// Start 1 second loop for bomb update function
if(!P3D.BombTimer)
	P3D.BombTimer = setInterval(() => P3D.BombUpdate(), 1000);