var WarrenDoorRotations = {
	"Rabbit": Vector3(0, -100, 0),
	"Fox": Vector3(38, -93, -68),
	"Bear": Vector3(-31.961, -323.628, -95.44901),
	"Deer": Vector3(-20.685, -87.46101, 89.103)
};

clearInterval(P3D.warrenInterval);

if(!P3D.warren) P3D.warren = {};


P3D.warrenLoop = function() {
	let warrenRoom = rooms["WarrenWoundCaverns"];
	
	for(let i in warrenRoom.clients) {
		let sclient = warrenRoom.clients[i];
		
		if(!valid(sclient)) continue;
		if(!sclient.warren) continue;
		
		let pillarsTrue = sclient.warren["PillarFox"] == 90 && sclient.warren["PillarBear"] == 90 && sclient.warren["PillarDeer"] == 270 && sclient.warren["PillarRabbit"] == 270;
		
		if(sclient.warren.goldCityOpen) {
			if(!pillarsTrue) {
				sclient.secretTranslate(
					`GoldCityBars (1)`, 
					{
						position: Vector3(-186.76, -141.1711, -110.75),
						speed: 1
					}
				);
				sclient.secretTranslate(
					`GoldCityBars`, 
					{
						position: Vector3(-155.99, -141.1711, -110.75),
						speed: 1
					}
				);
				sclient.warren.goldCityOpen = false;
			}
			continue;
		}
		
		if(pillarsTrue) {
			sclient.warren.goldCityOpen = true;
			
			sclient.chat("GOLD CITY OPENS!");
			
			sclient.secretTranslate(
				`GoldCityBars (1)`, 
				{
					position: Vector3(-207.9, -141.1711, -110.75),
					speed: 1
				}
			);
			sclient.secretTranslate(
				`GoldCityBars`, 
				{
					position: Vector3(-136.3, -141.1711, -110.75),
					speed: 1
				}
			);
		}
	}
}
P3D.warrenInterval = setInterval(P3D.warrenLoop, 1000);

P3D.checkHasAllKeys = function(client) {
	if(client.getItemCount("KeyRabbit") && client.getItemCount("KeyDeer") && client.getItemCount("KeyBear") && client.getItemCount("KeyFox")) {
		return true;
	}
	return false;
}

P3D.on("tool", function(client, tool) {
	if(client.roomId != "WarrenWoundCaverns" && client.roomId != "GoldCity") return;
	if(!client.warren) client.warren = {};
	
	let warrenRoom = rooms["WarrenWoundCaverns"];
	
	switch(tool) {
		case "ActivateDeer":
		case "ActivateBear":
		case "ActivateFox":
		case "ActivateRabbit": {
			PlaySFXAt("StoneButton", client.room, client.position);
			
			let buttonId = tool.split("Activate")[1];
			if(!buttonId) return;
			if(!client.warren[`loose${buttonId}`]) {
				client.notice(`The button is stuck. It needs to be loosened elsewhere.`);
				break;
			}
			client.secretTranslate(
				`Door${buttonId}`, 
				{
					localRotation: WarrenDoorRotations[buttonId],
					speed: 0.8
				}
			);
			break;
		}
		
		
		case "LoosenDeer":
		case "LoosenBear":
		case "LoosenFox":
		case "LoosenRabbit": {
			PlaySFXAt("StoneButton", client.room, client.position);
			
			let buttonId = tool.split("Loosen")[1];
			if(!buttonId) return;
			if(client.warren[`loose${buttonId}`]) break;
			
			client.warren[`loose${buttonId}`] = true;
			client.notice(`${buttonId} button loosened`);
			
			break;
		}
		
		case "KeyplateRabbit":
		case "KeyplateBear":
		case "KeyplateFox":
		case "KeyplateDeer": {
			let buttonId = tool.split("Keyplate")[1];
			if(!buttonId) break;
			
			if(client.getItemCount(`Key${buttonId}`) <= 0) {
				client.notice("You do not have the key.");
				break;
			}
			
			let warrenId = `Pillar${buttonId}`;
			
			if(client.warren[warrenId + "_ts"] && time() - client.warren[warrenId + "_ts"] < 5) {
				client.chat("The pillar is still turning");
				break;
			}
			
			if(!client.warren[warrenId]) {
				client.warren[warrenId] = 0;
			}
			
			client.warren[warrenId] += 90;
			
			if(client.warren[warrenId] >= 360) {
				client.warren[warrenId] = 0;
			}
			
			client.warren[warrenId + "_ts"] = time();
			
			client.secretTranslate(
				`GoldCityPillar${buttonId}`, 
				{
					localRotation: Vector3(0, client.warren[warrenId], 0),
					speed: 0.2
				}
			);
			
			PlaySFXAt("KeyLock", client.room, client.position);
			PlaySFXAt("StoneGrind", client.room, client.position);
			break;
		}
		
		
		
		case "RiddleRabbit": {
			if(!client.warren.looseRabbit) break;
			
			client.choice("Rabbit Riddle", ["Read Riddle", "Answer Riddle"], function(choice, client) {
				if(choice == "Read Riddle") {
					client.openBook("RiddleRabbit_{lang}");
				}else
				if(choice == "Answer Riddle") {
					client.input("Answer Rabbit Riddle", function(message, client) {
						let correct = ["lágrimas", "lagrimas", "tears"];
						
						let answer = String(message).toLowerCase().trim();
						
						if(answer.length > 100) return client.notice("Incorrect");
						
						for(let i in correct) {
							if(answer == correct[i]) {
								// CORRECT ANSWER!
								client.addItem("KeyRabbit", 1, true);
								P3D.checkHasAllKeys(client);
								return;
							}
						}
						
						return client.notice("Nothing happens");
					});
				}
			});
			break;
		}
		case "RiddleFox": {
			if(!client.warren.looseFox) break;
			
			client.choice("Fox Riddle", ["Read Riddle", "Answer Riddle"], function(choice, client) {
				if(choice == "Read Riddle") {
					client.openBook("RiddleFox_{lang}");
				}else
				if(choice == "Answer Riddle") {
					client.input("Answer Fox Riddle", function(message, client) {
						let correct = ["sabiduría", "sabiduria", "wisdom"];
						
						let answer = String(message).toLowerCase().trim();
						
						if(answer.length > 100) return client.notice("Incorrect");
						
						for(let i in correct) {
							if(answer == correct[i]) {
								// CORRECT ANSWER!
								client.addItem("KeyFox", 1, true);
								P3D.checkHasAllKeys(client);
								return;
							}
						}
						
						return client.notice("Nothing happens");
					});
				}
			});
			break;
		}
		case "RiddleBear": {
			if(!client.warren.looseBear) break;
			
			client.choice("Bear Riddle", ["Read Riddle", "Answer Riddle"], function(choice, client) {
				if(choice == "Read Riddle") {
					client.openBook("RiddleBear_{lang}");
				}else
				if(choice == "Answer Riddle") {
					client.input("Answer Bear Riddle", function(message, client) {
						let correct = ["echo", "eco", "an echo", "un eco"];
						
						let answer = String(message).toLowerCase().trim();
						
						if(answer.length > 100) return client.notice("Incorrect");
						
						for(let i in correct) {
							if(answer == correct[i]) {
								// CORRECT ANSWER!
								client.addItem("KeyBear", 1, true);
								P3D.checkHasAllKeys(client);
								return;
							}
						}
						
						return client.notice("Nothing happens");
					});
				}
			});
			break;
		}
		case "RiddleDeer": {
			if(!client.warren.looseDeer) break;
			
			client.choice("Deer Riddle", ["Read Riddle", "Answer Riddle"], function(choice, client) {
				if(choice == "Read Riddle") {
					client.openBook("RiddleDeer_{lang}");
				}else
				if(choice == "Answer Riddle") {
					client.input("Answer Deer Riddle", function(message, client) {
						let correct = ["forests", "forest", "bosque", "bosques", "a forest", "un bosque"];
						
						let answer = String(message).toLowerCase().trim();
						
						if(answer.length > 100) return client.notice("Incorrect");
						
						for(let i in correct) {
							if(answer == correct[i]) {
								// CORRECT ANSWER!
								client.addItem("KeyDeer", 1, true);
								P3D.checkHasAllKeys(client);
								return;
							}
						}
						
						return client.notice("Nothing happens");
					});
				}
			});
			break;
		}
		
		case "WoodenBBGun": {
			client.addItem("WoodenBBGun", 1, true);
			break;
		}
		case "ForestTomb": {
			client.openBook("RohimTomb");
			break;
		}
		case "TNTFill": {
			if(client.getItemCount("Gunpowder") < 1 || client.getItemCount("BallBearing") < 100) {
				client.notice("You need gunpowder and 100 ball bearings.");
				break;
			}
			if(!P3D.checkHasAllKeys(client)) {
				client.notice("You need to find all the keys first.");
				break;
			}
			client.removeItem("Gunpowder", 1);
			client.removeItem("BallBearing", 100);
			client.barrelFilled = true;
			break;
		}
		case "TNTFuse": {
			if(client.fuseLit) {
				//break;
			}
			if(!client.barrelFilled) {
				client.notice("You need to fill the barrel first.");
				break;
			}
			
			client.notice("Fuse LIT! RUN AWAY! RUN AwAY! RUNNNNN!");
			
			client.secretIns("Ins_Fire", "fusefire", Vector3(-0.3455, -28.465, 202.6272), Vector3(-90,0,0), Vector3(0.4,0.4,0.4));
			
			setTimeout(function() {
				client.send("unplace", ["fusefire"]);
				client.send("unplace", ["TNTBarrel"]);
				client.send("unplace", ["GCRock 1"]);
				client.send("unplace", ["GCRock 2"]);
				client.send("unplace", ["GCRock 3"]);
				client.secretIns("Explosion", "tntexp", Vector3(0.26, -24.11434, 205.73), Vector3(0,0,0), Vector3(20,20,20));
				client.setProp("wwcTnt", 1);
			}, 6000);
			
			client.fuseLit = true;
			break;
		}
		case "GoldCityGate": {
			if(client.warren.goldCityOpen) {
				client.joinRoom("GoldCity");
			}else{
				client.teleport(Vector3(0,0,0), Vector3(0,0,0));
			}
			break;
		}
		case "GCRoam": {
			if(client.superuser) break;
			if(client.prop("gcroam")) break;
			client.teleport(Vector3(-82.71701,28.37675,119.567), Vector3(0,0,0));
			break;
		}
		case "GCTomb": {
			client.openBook("RashadTomb");
			break;
		}
		case "GCBook": {
			client.openBook("GCBook");
			break;
		}
	}
	
}, "warrenwound");

P3D.on("roomJoined", function(client, roomId) {
	if(roomId == "GoldCity" && P3D.checkHasAllKeys(client) && !client.prop("gcboosted")) {
		setTimeout(function() {
			client.choice("Gold City Blessing", ["Extra Health", "Strength Boost", "Smokeless Eyes", "Damage Buff", "Light Fire", "None"], function(choice, client) {
				switch(choice) {
					case "Extra Health": {
						client.data.health += 100;
						mysql.query("UPDATE users SET health = health + 100 WHERE id = ?", [client.data.id]);
						client.notice("Your health has increased");
						client.updateMacros();
						break;
					}
					case "Strength Boost": {
						client.addStrength(500);
						break;
					}
					case "Smokeless Eyes": {
						client.addItem("SmokelessEyes", 1, true);
						break;
					}
					case "Light Fire": {
						client.setProp("g", "Fl");
						client.notice("You now cast light flames");
						break;
					}
					case "Damage Buff": {
						if(client.data.damageMultiplier <= 1)
							client.data.damageMultiplier = 1.05;
						else
							client.data.damageMultiplier += 0.05;
						
						mysql.query("UPDATE users SET damageMultiplier = ? WHERE id = ?", [client.data.damageMultiplier, client.data.id]);
						
						client.notice("Your damage has been increased");
						break;
					}
				}
				client.setProp("gcboosted", 1);
			});
		}, 3000);
	}
	
	if(roomId != "WarrenWoundCaverns") return;
	
	if(client.prop("wwcTnt")) {
		client.send("unplace", ["fusefire"]);
		client.send("unplace", ["TNTBarrel"]);
		client.send("unplace", ["GCRock 1"]);
		client.send("unplace", ["GCRock 2"]);
		client.send("unplace", ["GCRock 3"]);
	}
}, "warrenwound");